/* This is part of the netCDF package.
   Copyright 2006 University Corporation for Atmospheric Research/Unidata.
   See COPYRIGHT file for conditions of use.

   This is a very simple example which writes a netCDF file with
   Unicode names encoded with UTF-8. It is the NETCDF3 equivalent
   of tst_unicode.c

   $Id: tst_utf8.c,v 1.4 2008/04/22 15:32:08 russ Exp $
*/
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <netcdf.h>

/* The data file we will create. */
#define FILE7_NAME "tst_utf8.nc"
#define UNITS "units"
#define NDIMS 1
#define NX 18

int
main(int argc, char **argv)
{
   int ncid, dimid, varid;
   int dimids[NDIMS];

   /* UTF-8 encoding for Unicode 8-character "Hello" in Greek:
	CAPITAL LETTER KAPPA
	SMALL LETTER LAMBDA
	SMALL LETTER ALPHA
	SMALL LETTER ETA
	SMALL LETTER MU
	SMALL LETTER EPSILON WITH TONOS
	SMALL LETTER RHO
	SMALL LETTER ALPHA
   */

   unsigned char name_utf8[] = {
       0xCE, 0x9A,	  /* GREEK CAPITAL LETTER KAPPA  : 2-bytes utf8 */
       0xCE, 0xB1,	  /* GREEK SMALL LETTER LAMBDA   : 2-bytes utf8 */
       0xCE, 0xBB,	  /* GREEK SMALL LETTER ALPHA    : 2-bytes utf8 */
       0xCE, 0xB7,	  /* GREEK SMALL LETTER ETA      : 2-bytes utf8 */
       0xCE, 0xBC,	  /* GREEK SMALL LETTER MU       : 2-bytes utf8 */
       0xE1, 0xBD, 0xB3,  /* GREEK SMALL LETTER EPSILON
                             WITH TONOS                  : 3-bytes utf8 */
       0xCF, 0x81,	  /* GREEK SMALL LETTER RHO      : 2-bytes utf8 */
       0xCE, 0xB1, 0x00	  /* GREEK SMALL LETTER ALPHA    : 2-bytes utf8 */
   };

/* Name used for dimension, variable, and attribute value */
#define UNAME ((char *) name_utf8)
#define UNAMELEN (sizeof name_utf8)

   char name_in[UNAMELEN + 1], strings_in[UNAMELEN + 1];
   nc_type att_type;
   size_t att_len;


   /* Always check the return code of every netCDF function call. In
    * this example program, any retval which is not equal to NC_NOERR
    * (0) will cause the program to print an error message and exit
    * with a non-zero return code. */

   printf("*** creating UTF-8 test file %s...", FILE7_NAME);
   assert(nc_create(FILE7_NAME, NC_CLOBBER, &ncid)==0);

   /* Define dimension with Unicode UTF-8 encoded name */
   assert(nc_def_dim(ncid, UNAME, NX, &dimid)==0);
   dimids[0] = dimid;

   /* Define variable with same name */
   assert(nc_def_var(ncid, UNAME, NC_CHAR, NDIMS, dimids, &varid)==0);

   /* Create string attribute with same value */
   assert(nc_put_att_text(ncid, varid, UNITS, UNAMELEN, UNAME)==0);

   assert(nc_enddef(ncid)==0);

   /* Write string data, UTF-8 encoded, to the file */
   assert(nc_put_var_text(ncid, varid, UNAME)==0);

   assert(nc_close(ncid)==0);

   /* Check it out. */
   
   /* Reopen the file. */
   assert(nc_open(FILE7_NAME, NC_NOWRITE, &ncid)==0);
   assert(nc_inq_varid(ncid, UNAME, &varid)==0);
   assert(nc_inq_varname(ncid, varid, name_in)==0);
   assert(strncmp(UNAME, name_in, UNAMELEN) != 0==0);
   assert(nc_inq_att(ncid, varid, UNITS, &att_type, &att_len)==0);
   assert(att_type != NC_CHAR || att_len != UNAMELEN==0);
   assert(nc_get_att_text(ncid, varid, UNITS, strings_in)==0);
   strings_in[att_len] = '\0';
   assert(strncmp(UNAME, strings_in, UNAMELEN) != 0==0);
   assert(nc_close(ncid)==0);
   
   printf("*** SUCCESS writing example file tst_unicode.nc!\n");
   return 0;
}
